@@ -61,29 +61,6 @@ prioritize <- function(data, priority = NULL) {
     return(data)
   }
 
-  if (all(c("sector_ald", "sector_abcd") %in% names(data))) {
-
-    rlang::abort(
-      "too_many_sectors",
-      message = glue(
-        "Column `sector_ald` is deprecated as of r2dii.match 0.1.0, please use
-        `sector_abcd` instead."
-      )
-    )
-
-  } else if ("sector_ald" %in% names(data)) {
-
-    rlang::warn(
-      "deprecated_name",
-      message = glue(
-        "Column `sector_ald` is deprecated as of r2dii.match 0.1.0, please use
-        `sector_abcd` instead."
-      )
-    )
-
-    data <- dplyr::rename(data, sector_abcd = "sector_ald")
-  }
-
   data %>%
     check_crucial_names(
       c("id_loan", "level", "score", "sector", "sector_abcd")
